//------------------------------------------------------------------------------
#define USE_TEXTURE				1	// use 1 or 0 to use textured lookuptable
#define MONO_LOOKUPTABLE	0	// use 1 or 0 to select mono or color lookup table
//------------------------------------------------------------------------------


//#include <condefs.h>
//#include <conio.h>	// getch

#include <vtkRenderer.h>
#include <vtkRenderWindow.h>
#include <vtkRenderWindowInteractor.h>
#include <vtkPolyDataReader.h>
#include <vtkLookupTable.h>
#include <vtkLookupTableToTexture.h>
#include <vtkTexture.h>
#include <vtkTextureMapScalars.h>
#include <vtkDataSetMapper.h>
#include <vtkActor.h>
#include <vtkScalarBarActor.h>

#include <vtkTextureMapToPlane.h>
#include <vtkPNMReader.h>

#include <vtkPlaneSource.h>

#include <vtkTransformPolyDataFilter.h>
#include <vtkStructuredPoints.h>
#include <vtkPolyData.h>
#include <vtkTransform.h>
#include <vtkDoubleArray.h>
#include <vtkPointData.h>

//---------------------------------------------------------------------------
//USELIB("..\..\..\Tools\vtk\vtkbin\vtkdll\vtkdll.lib");
//USEUNIT("vtkTextureMapScalars.cpp");
//USEUNIT("vtkLookupTableToTexture.cpp");
//---------------------------------------------------------------------------

vtkRenderer *Renderer;
vtkRenderWindow *RenderWindow;
vtkRenderWindowInteractor *Interactor;
//int c;




//------------------------------------------------------------------------------
void Test1( void )
{
	int nDiv = 6;
	double range[2];
  range[0] = 0.6;
  range[1] = 1.8;

	vtkPolyDataReader *pdr1 = vtkPolyDataReader::New();
  	pdr1->SetFileName( "DigitisedRings.vtk" );
    pdr1->Update();

  vtkLookupTable *lut1 = vtkLookupTable::New();
		lut1->SetRange( range );
//	  if( USE_TEXTURE )
    	{
			lut1->SetNumberOfColors( nDiv );
    	}
    if( MONO_LOOKUPTABLE )
    	{
      // grey
      lut1->SetHueRange( 0, 0 );
      lut1->SetSaturationRange( 0, 0 );
      lut1->SetValueRange( 0.2, 0.8 );
      }
    else
    	{
      // red->blue
      lut1->SetHueRange( 0.0, 0.67 );
    	}
    lut1->Build();

  vtkLookupTableToTexture *ltt1 = vtkLookupTableToTexture::New();
    ltt1->SetLookupTable( lut1 );
  vtkTexture *tx1 = vtkTexture::New();
    tx1->SetInput( ltt1->GetOutput() );

  vtkTextureMapScalars *tms1 = vtkTextureMapScalars::New();
    tms1->SetInput( pdr1->GetOutput() );
    tms1->SetScalarRange( range );

  vtkDataSetMapper *pdm1 = vtkDataSetMapper::New();
	  if( USE_TEXTURE )
    	{
      pdm1->SetInput( tms1->GetOutput() );
      pdm1->ScalarVisibilityOff();
      }
    else
    	{
      pdm1->SetInput( pdr1->GetOutput() );
      pdm1->SetLookupTable( lut1 );
      pdm1->SetScalarRange( range );
      }

  vtkActor *a1 = vtkActor::New();
    a1->SetMapper( pdm1 );
	  if( USE_TEXTURE )
    	{
	    a1->SetTexture( tx1 );
    	}
  Renderer->AddActor( a1 );
//c = getch();

  // scale
  vtkScalarBarActor *sba1 = vtkScalarBarActor::New();
    sba1->SetLookupTable( lut1 );
    sba1->SetNumberOfLabels( nDiv+1 );
  Renderer->AddActor( sba1 );
//c = getch();

}

//------------------------------------------------------------------------------
void Test2( void )
{
	int nDiv = 10;
	double range[2];
// copied from expCos.cxx
  int i, numPts;
  double x[3];
  double r, deriv;

  // create plane to warp
  vtkPlaneSource *plane = vtkPlaneSource::New();
    plane->SetResolution (300,300);

  vtkTransform *transform = vtkTransform::New();
     transform->Scale(10.0,10.0,1.0);

  vtkTransformPolyDataFilter *transF = vtkTransformPolyDataFilter::New();
     transF->SetInput(plane->GetOutput());
     transF->SetTransform(transform);
     transF->Update();

  // compute Bessel function and derivatives. This portion could be
  // encapsulated into source or filter object.
  //
  vtkPolyData *input = transF->GetOutput();
  numPts = input->GetNumberOfPoints();

  vtkPoints *newPts = vtkPoints::New();
    newPts->SetNumberOfPoints(numPts);
//  vtkScalars *derivs = vtkScalars::New();
  vtkDoubleArray* derivs = vtkDoubleArray::New();
    //derivs->SetNumberOfScalars(numPts);
    derivs->SetNumberOfTuples(numPts);
  vtkPolyData *bessel = vtkPolyData::New();
    bessel->CopyStructure(input);
    bessel->SetPoints(newPts);
    bessel->GetPointData()->SetScalars(derivs);

  for (i=0; i<numPts; i++)
    {
    input->GetPoint(i,x);
    r = sqrt((double)x[0]*x[0] + x[1]*x[1]);
//    x[2] = exp(-r) * cos (10.0*r);
    newPts->SetPoint(i,x);
    deriv = -exp(-r) * (cos(10.0*r) + 10.0*sin(10.0*r));
    //derivs->SetScalar(i,deriv);
    derivs->SetTuple1(i,deriv);
    }
  newPts->Delete(); //reference counting - it's ok
  derivs->Delete();

  // get scalar range for lut etc
//  bessel->GetPointData()->GetScalars()->GetRange( range );
	range[0] = -10;
	range[1] =  10;

  vtkLookupTable *lut1 = vtkLookupTable::New();
		lut1->SetRange( range );
//	  if( USE_TEXTURE )
    	{
			lut1->SetNumberOfColors( nDiv );
    	}
    if( MONO_LOOKUPTABLE )
    	{
      // grey
      lut1->SetHueRange( 0, 0 );
      lut1->SetSaturationRange( 0, 0 );
      lut1->SetValueRange( 0.2, 0.8 );
      }
    else
    	{
      // red->blue
      lut1->SetHueRange( 0.0, 0.67 );
    	}
    lut1->Build();

  vtkLookupTableToTexture *ltt1 = vtkLookupTableToTexture::New();
    ltt1->SetLookupTable( lut1 );
  vtkTexture *tx1 = vtkTexture::New();
    tx1->SetInput( ltt1->GetOutput() );

  vtkTextureMapScalars *tms1 = vtkTextureMapScalars::New();
    tms1->SetInput( bessel );
    tms1->SetScalarRange( range );

  vtkDataSetMapper *pdm1 = vtkDataSetMapper::New();
	  if( USE_TEXTURE )
    	{
      pdm1->SetInput( tms1->GetOutput() );
      pdm1->ScalarVisibilityOff();
      }
    else
    	{
      pdm1->SetInput( bessel );
      pdm1->SetLookupTable( lut1 );
      pdm1->SetScalarRange( range );
      }

  vtkActor *a1 = vtkActor::New();
    a1->SetMapper( pdm1 );
	  if( USE_TEXTURE )
    	{
	    a1->SetTexture( tx1 );
    	}
  Renderer->AddActor( a1 );
//c = getch();

  // scale
  vtkScalarBarActor *sba1 = vtkScalarBarActor::New();
    sba1->SetLookupTable( lut1 );
    sba1->SetNumberOfLabels( nDiv+1 );
  Renderer->AddActor( sba1 );
}


//------------------------------------------------------------------------------
int main(int argc, char **argv)
{
  Renderer = vtkRenderer::New();
  RenderWindow = vtkRenderWindow::New();
	  RenderWindow->AddRenderer( Renderer );
  Interactor = vtkRenderWindowInteractor::New();
    Interactor->SetRenderWindow( RenderWindow );

//	Test1();	// two tree disc
  Test2();	// bessel

  // draw the resulting scene
  RenderWindow->Render();

//RenderWindow->SetFileName( "DigitisedRings1.ppm" );
//RenderWindow->SaveImageAsPPM();

  // Begin mouse interaction
  Interactor->Start();

//RenderWindow->SetFileName( "DigitisedRings2.ppm" );
//RenderWindow->SaveImageAsPPM();
  return 0;
}



