/*=========================================================================

  Program:   Visualization Toolkit
  Module:    $RCSfile: vtkTextureMapScalars.cxx,v $
  Language:  C++
  Date:      $Date: 2000/02/04 17:07:13 $
  Version:   $Revision: 1.35 $
  Thanks:    Thanks to David Pont at Forest Research, New Zealand who
             developed and contributed this class.


Copyright (c) 1993-2003 Ken Martin, Will Schroeder, Bill Lorensen 
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice,
   this list of conditions and the following disclaimer.

 * Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

 * Neither name of Ken Martin, Will Schroeder, or Bill Lorensen nor the names
   of any contributors may be used to endorse or promote products derived
   from this software without specific prior written permission.

 * Modified source versions must be plainly marked as such, and must not be
   misrepresented as being the original software.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

=========================================================================*/
#include "vtkTextureMapScalars.h"
#include "vtkMath.h"
//#include "vtkTCoords.h"
#include "vtkObjectFactory.h"
#include "vtkLookupTable.h"
#include "vtkFloatArray.h"
#include "vtkPointData.h"
#include "vtkCellData.h"

#include <vtkTIFFWriter.h>	// !!!
//#include <vtkBMPWriter.h>	// !!!
#include <vtkImageData.h>	// !!!
//------------------------------------------------------------------------------
vtkTextureMapScalars* vtkTextureMapScalars::New()
{
  // First try to create the object from the vtkObjectFactory
  vtkObject* ret = vtkObjectFactory::CreateInstance("vtkTextureMapScalars");
  if(ret)
    {
    return (vtkTextureMapScalars*)ret;
    }
  // If the factory was unable to create the object, then create it here.
  return new vtkTextureMapScalars;
}


// Construct with s,t range=(0,1)
vtkTextureMapScalars::vtkTextureMapScalars()
{
  this->ScalarRange[0] = 0.0;
  this->ScalarRange[1] = 1.0;

  this->SRange[0] = 0.0;
  this->SRange[1] = 1.0;

  this->TRange[0] = 0.0;
  this->TRange[1] = 1.0;
}

void vtkTextureMapScalars::Execute()
{
// !!! Use t coord from input t coords if there (switchable)?
	// generate texture coordinates
  double tcoords[3];
  int numPts;
  //vtkTCoords *newTCoords, *currentTCoords;
  vtkFloatArray *newTCoords, *currentTCoords;
  int i;
  double s, sRange, scalarRange;
  vtkDataSet *input = this->GetInput();
  vtkDataSet *output = this->GetOutput();
  numPts = input->GetNumberOfPoints();
//2	double bounds[6], point[3], x[3], y[3], z[3], len, dot[3];

  //currentTCoords = input->GetPointData()->GetTCoords();

  vtkDebugMacro(<<"Generating texture coordinates!");

  // First, copy the input to the output as a starting point
  output->CopyStructure( input );

  //  Allocate texture coordinate data
//  newTCoords = vtkTCoords::New();
//  newTCoords->SetNumberOfTCoords(numPts);
  newTCoords = vtkFloatArray::New();
  newTCoords->SetNumberOfComponents(2);
  newTCoords->SetNumberOfTuples(numPts);
  newTCoords->SetName("TCoords");


//1	newTCoords = input->GetPointData()->GetTCoords();

  sRange = (this->SRange[1] - this->SRange[0]);
//  tRange = (this->TRange[1] - this->TRange[0]);
  scalarRange = (this->ScalarRange[1] - this->ScalarRange[0]);

	tcoords[1] = 0;
	tcoords[2] = 0;

  //  Now can loop over all points, computing texture coordinates.
  for (i=0; i<numPts; i++)
    {
// utilise existing tcoords, should check if there, and have user switch enable their use
//currentTCoords->GetTCoord( i, tcoords );

    // map scalar value into s texture coordinate
    //s = input->GetPointData()->GetScalars()->GetScalar( i ) - this->ScalarRange[0];
    s = input->GetPointData()->GetScalars()->GetTuple1( i ) - this->ScalarRange[0];
    tcoords[0] = this->SRange[0] + sRange*( s / scalarRange );	// !!! check

    //newTCoords->SetTCoord( i, tcoords );
    newTCoords->SetTuple( i, tcoords );
    }

  // Update ourselves
  output->GetPointData()->CopyTCoordsOff();
//1	output->GetPointData()->CopyTCoordsOn();
  output->GetPointData()->PassData(input->GetPointData());
  output->GetCellData()->PassData(input->GetCellData());
  output->GetPointData()->SetTCoords(newTCoords);
  newTCoords->Delete();
}

void vtkTextureMapScalars::PrintSelf(ostream& os, vtkIndent indent)
{
  vtkDataSetToDataSetFilter::PrintSelf(os,indent);
  //!!! update

  os << indent << "Scalar Range: (" << this->ScalarRange[0] << ", "
                                    << this->ScalarRange[1] << ")\n";

  os << indent << "S Range: (" << this->SRange[0] << ", "
                               << this->SRange[1] << ")\n";
  os << indent << "T Range: (" << this->TRange[0] << ", "
                               << this->TRange[1] << ")\n";

}

